#!/usr/bin/env bash

./qt5-mac.sh $1 \
	https://download.qt.io/online/qtsdkrepository/mac_x64/desktop/qt5_595/qt.595.clang_64/5.9.5-0-201804111039qtbase-MacOS-MacOS_10_12-Clang-MacOS-MacOS_10_12-X86_64.7z \
	https://download.qt.io/online/qtsdkrepository/mac_x64/desktop/qt5_595/qt.595.clang_64/5.9.5-0-201804111039qttools-MacOS-MacOS_10_12-Clang-MacOS-MacOS_10_12-X86_64.7z \
	https://download.qt.io/online/qtsdkrepository/mac_x64/desktop/qt5_595/qt.595.qtwebengine.clang_64/5.9.5-0-201804111039qtwebengine-MacOS-MacOS_10_12-Clang-MacOS-MacOS_10_12-X86_64.7z \
	https://download.qt.io/online/qtsdkrepository/mac_x64/desktop/qt5_595/qt.595.clang_64/5.9.5-0-201804111039qtwebchannel-MacOS-MacOS_10_12-Clang-MacOS-MacOS_10_12-X86_64.7z \
	https://download.qt.io/online/qtsdkrepository/mac_x64/desktop/qt5_595/qt.595.clang_64/5.9.5-0-201804111039qtdeclarative-MacOS-MacOS_10_12-Clang-MacOS-MacOS_10_12-X86_64.7z \
	https://download.qt.io/online/qtsdkrepository/mac_x64/desktop/qt5_595/qt.595.clang_64/5.9.5-0-201804111039qtquickcontrols2-MacOS-MacOS_10_12-Clang-MacOS-MacOS_10_12-X86_64.7z \
	https://download.qt.io/online/qtsdkrepository/mac_x64/desktop/qt5_595/qt.595.clang_64/5.9.5-0-201804111039qtquickcontrols-MacOS-MacOS_10_12-Clang-MacOS-MacOS_10_12-X86_64.7z \
	https://download.qt.io/online/qtsdkrepository/mac_x64/desktop/qt5_595/qt.595.clang_64/5.9.5-0-201804111039qtlocation-MacOS-MacOS_10_12-Clang-MacOS-MacOS_10_12-X86_64.7z \
	https://download.qt.io/online/qtsdkrepository/mac_x64/desktop/qt5_595/qt.595.clang_64/5.9.5-0-201804111039qtconnectivity-MacOS-MacOS_10_12-Clang-MacOS-MacOS_10_12-X86_64.7z