static const char key[] = "-----BEGIN PUBLIC KEY-----\n"
    "MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEA265O2NgcDI5hk1n90xzH\n"
    "P7nZU+sSQoJcpOVd71bnqhjTyrmEHk9gJzyB0O8OJPlNonE16lQfM+a0ncH+XLt7\n"
    "HKF73ZwsPCHg1MX/lGpSQtfC3jUGZdFssGhKBo06Thtp8coptRiRl2ZAtvgPNKME\n"
    "NZLHzck+tI15UwKkKfUL3zfxgapJEPfd7XKT7NCj+n9fa3MXFnSIiDjMvb6f2oeC\n"
    "YPpE7pqAaSgde/ZeveJGWgexHzOgA27nz+LuBo3ITYp3HFu9Q0xymQs0fxUFazw+\n"
    "YIRJZD5yNNitPOsrRrYgaprNrVS6AvB7lhauMjz+R/r2OdbaszO1Gi9wqGWGoIcy\n"
    "VfBCWf75SNwanf8/V5RiQP32bKFiHPlQkqMjrrS3FwPqbXwj/W0rFGgy6GolrFIJ\n"
    "rdWeaLSgIY5M3qhowArB7aTMPZItQhOIPqZgTt79GCsa4zKrBvFr7TM7dTRVbLzG\n"
    "2yA/cS88sQpnsO/Li/oNunG7Z1DRmWc3otUtsVVvmxFT5p62aoGEpFvbDQuel2yO\n"
    "o8VDGK1klqEzRvH7SGLA8VLyvk3gwhXRJxep6l5LLiPXSVw7DqNJ+QhtsCPkteBU\n"
    "sVdpnLa3TQFB8Ztn/ZC0L/foupvn95OF4bC6/Owc16kQjSs9V+GgtzGTGxjuGR96\n"
    "kKc4nu1CZCwxqHpRpquQIu8CAwEAAQ==\n"
    "-----END PUBLIC KEY-----";